@@ -1,5 +1,7 @@
 pub mod layers;
 
+use std::path::{Path, PathBuf};
+
 use thiserror::Error as ThisError;
 
 pub use layeredconf_derive::LayeredConf;
@@ -12,8 +14,22 @@ pub enum Error {
     SolidifyFailedMissing { missing: Vec<String> },
     #[error("Solidify failed, no layers")]
     SolidifyFailedNoLayers,
-    #[error("I/O Error {wrapped:?}")]
-    IoError { wrapped: std::io::Error },
+    #[error("Unknown extension {extension:?}")]
+    UnknownExtension { extension: Option<String> },
+    #[error("Auto format detection failed")]
+    AutoFormatFailed,
+    #[error("Loop detected loading config files")]
+    LoopingLoadConfig,
+    #[error("Path canonicalization error {wrapped:?} for {path:?}")]
+    PathCanonicalization {
+        wrapped: std::io::Error,
+        path: PathBuf,
+    },
+    #[error("I/O Error {wrapped:?} for {path:?}")]
+    IoError {
+        wrapped: std::io::Error,
+        path: PathBuf,
+    },
     #[error("Json Error {wrapped:?}")]
     JsonError { wrapped: serde_json::Error },
     #[error("Toml Error {wrapped:?}")]
@@ -22,9 +38,17 @@ pub enum Error {
     YamlError { wrapped: serde_yaml::Error },
 }
 
-impl From<std::io::Error> for Error {
-    fn from(wrapped: std::io::Error) -> Self {
-        Error::IoError { wrapped }
+pub(crate) fn map_io_error(path: &'_ Path) -> impl Fn(std::io::Error) -> Error + '_ {
+    move |wrapped| Error::IoError {
+        wrapped,
+        path: path.to_path_buf(),
+    }
+}
+
+pub(crate) fn map_canonicalization_error(path: &'_ Path) -> impl Fn(std::io::Error) -> Error + '_ {
+    move |wrapped| Error::IoError {
+        wrapped,
+        path: path.to_path_buf(),
     }
 }
 
@@ -52,6 +76,8 @@ pub trait LayeredConfSolid {
 
 pub trait LayeredConfLayer {
     type Config: LayeredConfSolid + serde::de::DeserializeOwned;
+
+    fn load_configs(&self) -> Vec<PathBuf>;
 }
 
 pub trait LayeredConfMerge<TLayer> {
